@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2003 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -74,7 +74,7 @@
 #ifdef COMPILER
 #include "freePreNetFullCompiler.cc"
 #endif
-
+    
 FreePreNet::FreePreNet(bool expandRemainderNodes)
   : expandRemainderNodes(expandRemainderNodes)
 {
@@ -83,7 +83,7 @@ FreePreNet::FreePreNet(bool expandRemainderNodes)
 void
 FreePreNet::buildNet(FreeSymbol* symbol)
 {
-  //cout << "buildNet for " << symbol << endl;
+  DebugEnter("for " << symbol);
   topSymbol = symbol;
   nrFailParents = 0;
   nrFailVisits = 0;
@@ -91,9 +91,7 @@ FreePreNet::buildNet(FreeSymbol* symbol)
   int nrEquations = equations.length();
   if (nrEquations == 0)
     return;
-
-  //cout << "nrEquations = " << nrEquations << endl;
-
+  DebugInfo("nrEquations = " << nrEquations);
   patterns.expandTo(nrEquations);
   LiveSet liveSet;
   LiveSet potentialSubsumers;
@@ -103,9 +101,9 @@ FreePreNet::buildNet(FreeSymbol* symbol)
       Equation* e = equations[i];
       Term* p = e->getLhs();
       patterns[i].term = p;
-      FOR_EACH_CONST(j, LiveSet, potentialSubsumers)
+      for (int j : potentialSubsumers)
 	{
-	  if (patterns[*j].term->subsumes(p, false))
+	  if (patterns[j].term->subsumes(p, false))
 	    {
 	      DebugAdvisory(e << " subsumed");
 	      goto subsumedAtTop;
@@ -142,7 +140,7 @@ FreePreNet::buildNet(FreeSymbol* symbol)
   NatSet positionsTested;
   (void) makeNode(liveSet, fringe, positionsTested);
   // dump(cerr, 0);
-  //cout << "buildNet for " << symbol << " done; nodes = " << net.size() << endl;
+  DebugExit("for " << symbol << " done; nodes = " << net.size());
 }
 
 int
@@ -150,7 +148,7 @@ FreePreNet::makeNode(const LiveSet& liveSet,
 		     const NatSet& reducedFringe,
 		     const NatSet& positionsTested)
 {
-  //cerr << "liveSet has " << liveSet.size() << " patterns" << endl;
+  DebugEnter("liveSet has " << liveSet.size() << " patterns");
   //
   //	First check to see if there is already a node with the same set
   //	of live patterns and the same reduced fringe that we can share.
@@ -214,7 +212,7 @@ FreePreNet::makeNode(const LiveSet& liveSet,
       //
       //	Match node case; choose a test position.
       //
-      int positionIndex = findBestPosition(key, n);
+      int positionIndex = findBestPosition(key, n);  // fills out n.sons
       NatSet newPositionsTested(positionsTested);
       newPositionsTested.insert(positionIndex);
       n.testPositionIndex = positionIndex;
@@ -223,8 +221,8 @@ FreePreNet::makeNode(const LiveSet& liveSet,
       //	prevents a quadratic blow up on certain examples with
       //	huge pattern set.
       //
-      int totalSymbols = topSymbol->getModule()->getSymbols().size();
-      Vector<LiveSet> liveSets(totalSymbols);
+      Index nrArcs = n.sons.size();
+      Vector<LiveSet> liveSets(nrArcs);
       LiveSet defaultLiveSet;
       partitionLiveSet(liveSet,
 		       positionIndex,
@@ -234,10 +232,10 @@ FreePreNet::makeNode(const LiveSet& liveSet,
       //
       //	Make an arc for each symbol we must check for.
       //
-      int nrSymbols = n.sons.length();
-      for (int i = 0; i < nrSymbols; i++)
+      for (Index i = 0; i < nrArcs; ++i)
 	{
-	  Symbol* symbol = n.sons[i].label;
+	  Arc& son = n.sons[i];
+	  Symbol* symbol = son.label;
 	  //
 	  //	Make a new fringe:
 	  //	(1) Remove tested position from old fringe.
@@ -251,16 +249,14 @@ FreePreNet::makeNode(const LiveSet& liveSet,
 	  newFringe.subtract(positionIndex);
 	  expandFringe(positionIndex, symbol, newFringe);
 	  LiveSet newLiveSet;
-	  findLiveSet(liveSets[symbol->getIndexWithinModule()],
+	  findLiveSet(liveSets[i],
 		      positionIndex,
 		      symbol, 
 		      newFringe,
 		      newLiveSet);
 	  reduceFringe(newLiveSet, newFringe);
-	  Assert(!(newLiveSet.empty()),
-		 "empty live set on arc labeled with symbol");
-	  int t = makeNode(newLiveSet, newFringe, newPositionsTested);
-	  n.sons[i].target = t;
+	  Assert(!(newLiveSet.empty()), "empty live set on arc labeled with symbol");
+	  son.target = makeNode(newLiveSet, newFringe, newPositionsTested);
 	}
       //
       //	Make an arc for the default case.
@@ -308,31 +304,33 @@ FreePreNet::expandFringe(int positionIndex, Symbol* symbol, NatSet& fringe)
 void
 FreePreNet::reduceFringe(const LiveSet& liveSet, NatSet& fringe) const
 {
+  NatSet useless;
   //
   //	For each position in the fringe.
   //
-  const NatSet::const_iterator ei = fringe.end();
-  for (NatSet::const_iterator i = fringe.begin(); i != ei; ++i)
+  for (int i : fringe)
     {
-      const Vector<int>& position = positions.index2Position(*i);
+      const Vector<int>& position = positions.index2Position(i);
       //
       //	See if at least one live pattern has a stable symbol at this position.
       //
-      FOR_EACH_CONST(j, LiveSet, liveSet)
+      for (int j : liveSet)
 	{
-	  if (FreeTerm* f = dynamic_cast<FreeTerm*>(patterns[*j].term))
+	  Term* p = patterns[j].term;
+	  if (typeid(*p) == typeid(FreeTerm))
 	    {
-	      if (Term* t = f->locateSubterm(position))
+	      if (Term* t = static_cast<FreeTerm*>(p)->locateSubterm(position))
 		{
 		  if (t->stable())
 		    goto survive;
 		}
 	    }
 	}
-      fringe.subtract(*i);  // eliminate useless pattern from fringe
+      useless.insert(i);  // useless position
     survive:
       ;
     }
+  fringe.subtract(useless);
 }
 
 void
@@ -343,10 +341,9 @@ FreePreNet::findLiveSet(const LiveSet& original,
 			LiveSet& liveSet)
 {
   const Vector<int> position(positions.index2Position(positionIndex));  // deep copy - because ref become stale
-  
-  FOR_EACH_CONST(i, LiveSet, original)
+
+  for (int patternIndex : original)
     {
-      int patternIndex = *i;
       if (FreeTerm* f = dynamic_cast<FreeTerm*>(patterns[patternIndex].term))
 	{
 	  if (Term* t = f->locateSubterm(position))
@@ -388,16 +385,22 @@ FreePreNet::partitionLiveSet(const LiveSet& original,
 			     LiveSet& defaultLiveSet)
 {
   const Vector<int>& position = positions.index2Position(positionIndex);  // safe because we won't add new positions
-  
-  FOR_EACH_CONST(i, LiveSet, original)
+  //
+  //	We want to efficiently map symbols to arc indices in the case there are many arcs.
+  //
+  map<Symbol*, Index> arcIndexLookup;
+  Index nrArcs = arcs.size();  // number of symbols we need to consider
+  for (Index i = 0; i < nrArcs; ++i)
+    arcIndexLookup[arcs[i].label] = i;
+
+  for (int patternIndex : original)
     {
-      int patternIndex = *i;
       if (FreeTerm* f = dynamic_cast<FreeTerm*>(patterns[patternIndex].term))
 	{
 	  if (Term* t = f->locateSubterm(position))
 	    {
 	      if (t->stable())
-		liveSets[t->symbol()->getIndexWithinModule()].insert(patternIndex);
+		liveSets[arcIndexLookup[t->symbol()]].insert(patternIndex);
 	      else
 		{
 		  //
@@ -409,11 +412,11 @@ FreePreNet::partitionLiveSet(const LiveSet& original,
 		  //	Need to consider pattern for the live set of each active
 		  //	symbol that could match our unstable subpattern.
 		  //
-		  FOR_EACH_CONST(j, Vector<Arc>, arcs)
+		  for (Index i = 0; i < nrArcs; ++i)
 		    {
-		      Symbol* symbol = j->label;
+		      Symbol* symbol = arcs[i].label;
 		      if (symbol->mightMatchPattern(t))
-			liveSets[symbol->getIndexWithinModule()].insert(patternIndex);
+			liveSets[i].insert(patternIndex);
 		    }
 		}
 	      continue;
@@ -424,8 +427,8 @@ FreePreNet::partitionLiveSet(const LiveSet& original,
       //	Either way we add it to all live sets.
       //
       defaultLiveSet.insert(patternIndex);
-      FOR_EACH_CONST(j, Vector<Arc>, arcs)
-	liveSets[j->label->getIndexWithinModule()].insert(patternIndex);
+      for (LiveSet& l : liveSets)
+	l.insert(patternIndex);
     }
 }
 
@@ -437,11 +440,11 @@ FreePreNet::partiallySubsumed(const LiveSet& liveSet,
   if (liveSet.empty())
     return false;
   Term* vp = patterns[victim].term;
-  FOR_EACH_CONST(i, LiveSet, liveSet)
+  for (int i : liveSet)
     {
-      Assert(*i < victim, "current liveSet contains pattern >= victim");
-      if ((patterns[*i].flags & SUBSUMER) &&
-	  subsumesWrtReducedFringe(patterns[*i].term, vp, topPositionIndex, fringe))
+      Assert(i < victim, "current liveSet contains pattern >= victim");
+      if ((patterns[i].flags & SUBSUMER) &&
+	  subsumesWrtReducedFringe(patterns[i].term, vp, topPositionIndex, fringe))
 	return true;
     }
   return false;
@@ -473,9 +476,9 @@ FreePreNet::findBestPosition(const NodeIndex& ni, NodeBody& n) const
       //	Gather the set of symbols that occur at this position
       //	in a live pattern.
       //
-      FOR_EACH_CONST(j, LiveSet, ni.liveSet)
+      for (int j : ni.liveSet)
 	{
-	  if (FreeTerm* f = dynamic_cast<FreeTerm*>(patterns[*j].term))
+	  if (FreeTerm* f = dynamic_cast<FreeTerm*>(patterns[j].term))
 	    {
 	      Term* t = f->locateSubterm(path);
 	      if (t != 0 && t->stable())
@@ -483,7 +486,7 @@ FreePreNet::findBestPosition(const NodeIndex& ni, NodeBody& n) const
 		  ++nrStable;
 		  symbols.insert(t->symbol());
 		}
-	      else if (nrStable == 0 && patterns[*j].flags & SUBSUMER)
+	      else if (nrStable == 0 && patterns[j].flags & SUBSUMER)
 		++nrUnstableSubsumersAbove;
 	    }
 	  ++nrLive;
@@ -677,13 +680,13 @@ FreePreNet::dumpLiveSet(ostream& s, const LiveSet& liveSet)
   else
     {
       bool first = true;
-      FOR_EACH_CONST(j, LiveSet, liveSet)
+      for (int j : liveSet)
 	{
 	  if (first)
 	    first = false;
 	  else
 	    s << ", ";
-	  s << *j;
+	  s << j;
 	}
     }
 }
