@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2024 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2025 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -56,8 +56,12 @@
 //	strategy class definitions
 #include "callStrategy.hh"
 
-ImportTranslation::ImportTranslation(ImportModule* target, Renaming* renaming, bool preserveVariableIndicesFlag)
-  : preserveVariableIndicesFlag(preserveVariableIndicesFlag)
+ImportTranslation::ImportTranslation(ImportModule* target,
+				     Renaming* renaming,
+				     bool preserveVariableIndicesFlag,
+				     bool liftVariablesToKind)
+  : preserveVariableIndicesFlag(preserveVariableIndicesFlag),
+    liftVariablesToKind(liftVariablesToKind)
 {
   push(renaming, target);
 }
@@ -114,6 +118,8 @@ ImportTranslation::translate(Symbol* symbol)
     case SymbolType::VARIABLE:
       {
 	Sort* sort = translate(safeCast(VariableSymbol*, symbol)->getSort());
+	if (liftVariablesToKind)
+	  sort = sort->component()->sort(Sort::KIND);
 	s = targets.back()->instantiateVariable(sort);
 	break;
       }
@@ -167,7 +173,7 @@ ImportTranslation::translateStrategy(RewriteStrategy* strat,
   const Vector<Sort*>& domain = strat->getDomain();
   Vector<int> sortNames(nrArgs);
   for (int i = 0; i < nrArgs; i++)
-    sortNames[i] = domain[i]->id();
+    sortNames[i] = domain[i]->component()->sort(Sort::FIRST_USER_SORT)->id();
 
   for (RenamingList::const_iterator i = renamings.begin(), e = renamings.end(); i != e; ++i)
     {
@@ -356,17 +362,14 @@ ImportTranslation::translateRegularSymbol(Symbol* symbol,
   int id = symbol->id();
   Vector<int> sortNames(nrArgs + 1);
   for (int i = 0; i < nrArgs; ++i)
-    sortNames[i] = symbol->domainComponent(i)->sort(1)->id();
-  sortNames[nrArgs] = symbol->rangeComponent()->sort(1)->id();
+    sortNames[i] = symbol->domainComponent(i)->sort(Sort::FIRST_USER_SORT)->id();
+  sortNames[nrArgs] = symbol->rangeComponent()->sort(Sort::FIRST_USER_SORT)->id();
 
   for (RenamingList::const_iterator i = renamings.begin(), e = renamings.end(); i != e; ++i)
     {
       Renaming* r =  *i;
       if (r != 0)
 	{
-	  //
-	  //	Translate name.
-	  //
 	  int index = r->renameOp(id, sortNames);
 	  if (index != NONE)
 	    {
@@ -441,12 +444,12 @@ ImportTranslation::findTargetVersionOfSymbol(Symbol* symbol)
   Vector<ConnectedComponent*> domainComponents(nrArgs);
   for (int i = 0; i < nrArgs; ++i)
     {
-      Sort* ds = symbol->domainComponent(i)->sort(1);
+      Sort* ds = symbol->domainComponent(i)->sort(Sort::FIRST_USER_SORT);
       Sort* targetSort = target->findSort(ds->id());
       Assert(targetSort != 0, "couldn't find sort " << ds << " in " << target);
       domainComponents[i] = targetSort->component();
     }
-  Sort* rs = symbol->rangeComponent()->sort(1);
+  Sort* rs = symbol->rangeComponent()->sort(Sort::FIRST_USER_SORT);
   Sort* targetSort = target->findSort(rs->id());
   Assert(targetSort != 0, "couldn't find sort  " << rs << " in " << target);
   ConnectedComponent* rangeComponent = targetSort->component();
