@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2021 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -117,9 +117,15 @@ UserLevelRewritingContext::setHandlers(bool handleCtrlC)
   //	Stack overflows are reported as SIGSEGV signals and so we need to use the
   //	libsigsegv library to heuristically distinguish the two conditions.
   //
-  static char altStack[SIGSTKSZ];
+  size_t altStackSize = SIGSTKSZ;  // might not be a compile time constant
+  void* altStack = malloc(altStackSize);
+  if (altStack == 0)
+    {
+      cerr << "Fatal error: unable to allocate alternative stack." << endl;
+      exit(OUT_OF_MEMORY);
+    }
   sigsegv_install_handler(sigsegvHandler);  // illegal memory access or stack overflow
-  stackoverflow_install_handler(stackOverflowHandler, altStack, sizeof(altStack));
+  stackoverflow_install_handler(stackOverflowHandler, altStack, altStackSize);
 #else
   //
   //	If we can't use the library we will will catch SIGSEGVs but not install
@@ -154,7 +160,7 @@ UserLevelRewritingContext::stackOverflowHandler(int emergency, stackoverflow_con
   //
   static char message[] = "\nFatal error: stack overflow.\n\
 This can happen because you have an infinite computation, say a runaway\n\
-recursion, or model checking an infinite model. It can also happen because\n\
+recursion, or are model checking an infinite model. It can also happen because\n\
 the stacksize limit in your environment is set too low for the computation\n\
 you are trying to do. You can find the value of your stacksize with the\n\
 tcsh command 'limit stacksize' or the bash command 'ulimit -s'.\n\
@@ -163,7 +169,7 @@ Depending on your operating system configuration you may be able to\n\
 increase your stacksize with the tcsh command 'unlimit stacksize'\n\
 or the bash command 'ulimit -s unlimited'.\n\n";
   returnValueDump = write(STDERR_FILENO, message, sizeof(message) - 1);
-  _exit(STACK_OVERFLOW);  // don't call atexit() functions with a bad machine state
+  exit(STACK_OVERFLOW);  // calling _exit() loses the message on Macs
 }
 
 int
@@ -191,7 +197,7 @@ runtime to the bug being visible is greater than 10 seconds.\n\n";
   returnValueDump = write(STDERR_FILENO, message1, sizeof(message1) - 1);
   returnValueDump = write(STDERR_FILENO, message2, sizeof(message2) - 1);
   returnValueDump = write(STDERR_FILENO, message3, sizeof(message3) - 1);
-  _exit(INTERNAL_ERROR);  // don't call atexit() functions with a bad machine state
+  exit(INTERNAL_ERROR);  // calling _exit() loses the message on Macs
 }
 
 void
@@ -366,11 +372,12 @@ UserLevelRewritingContext::commandLoop()
       cout.flush();
       parseResult = NORMAL;
       ioManager.startCommand();
-      bool parseError = yyparse(&parseResult);
+      int parseError = yyparse(&parseResult);
       //cout << "parser result is: " << parseError << endl;
       if (parseError || ctrlC_Flag)
 	{
-	  cout << '\n';  // terminate any partially typed in line
+	  if (interactiveFlag)
+	    ++lineNumber;  // assume ^C or ^D and treat it as starting a new line
 	  setTraceStatus(interpreter.getFlag(Interpreter::EXCEPTION_FLAGS));
 	  cleanUpParser();
 	  cleanUpLexer();
@@ -397,19 +404,13 @@ UserLevelRewritingContext::commandLoop()
 		//
 		//	This is the only normal exit() that can be taken.
 		//
-		cout << "Bye.\n";
-#ifndef NO_ASSERT
+		interpreter.quit();
+#ifndef FAST_QUIT
 		//
-		//	Look for memory bugs.
+		//	Look for memory bugs by deleting everything. Not needed under normal
+		//	circumstances because we're about to exit.
 		//
 		delete &interpreter;
-#else
-		//
-		//	If we are not going to delete the interpreter
-		//	in the interests of a quick exit, at least clean
-		//	up any XML log we may have started.
-		//
-		interpreter.endXmlLog();
 #endif
 		exit(NORMAL_EXIT);
 	      }
