@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2003 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -76,6 +76,7 @@
 #include "viewExpression.hh"
 #include "moduleExpression.hh"
 #include "interpreter.hh"
+#include "maudeLatexBuffer.hh"
 
 //	our stuff
 #include "execute.cc"
@@ -90,12 +91,13 @@
 #include "srewrite.cc"
 
 Interpreter::Interpreter()
+: PrintSettings(DEFAULT_PRINT_FLAGS)
 {
   xmlLog = 0;
   xmlBuffer = 0;
+  latexBuffer = 0;
 
   flags = DEFAULT_FLAGS;
-  printFlags = DEFAULT_PRINT_FLAGS;
   currentModule = 0;
   currentView = 0;
 
@@ -120,6 +122,43 @@ Interpreter::~Interpreter()
   delete xmlLog;
 }
 
+void
+Interpreter::outputBanner(const char* date, const char* time, time_t seconds)
+{
+  if (latexBuffer)
+    latexBuffer->generateBanner(date, time, seconds);
+}
+
+void
+Interpreter::quit()
+{
+  ostream* latexStream = latexBuffer ? &(latexBuffer->getStream()) : nullptr;
+  if (latexStream)
+    *latexStream << "%\n%  quit\n%\n\\begin{maudeResultParagraph}\n";
+  MemoryCell::maybeShowResources(cout, latexStream);
+  cout << "Bye.\n";
+  if (latexStream)
+    *latexStream << "\\par\\maudeResponse{Bye.}\n\\end{maudeResultParagraph}\n%\n%  End of quit\n%\n";
+  endXmlLog();
+  endLatexLog();
+}
+
+void
+Interpreter::cleanCaches()
+{
+  //
+  //	We can have constructed modules that are users of constructed modules
+  //	and/or view instantiations.
+  //	We can also have view instantiations that are user of constructed
+  //	modules and/or view instantiations.
+  //	So we iterate the purge up to fixed point.
+  //	This is expensive for long dependency chains but in practice long
+  //	dependency chains rare.
+  //
+  while (destructUnusedModules() + destructUnusedViews() > 0)
+    ;
+}
+
 void
 Interpreter::setFlag(Flags flag, bool polarity)
 {
@@ -130,15 +169,6 @@ Interpreter::setFlag(Flags flag, bool polarity)
   RewritingContext::setTraceStatus(flags & EXCEPTION_FLAGS);
 }
 
-void
-Interpreter::setPrintFlag(PrintFlags flag, bool polarity)
-{
-  if (polarity)
-    printFlags |= flag;
-  else
-    printFlags &= ~flag;
-}
-
 void
 Interpreter::showProfile() const
 {
@@ -163,6 +193,20 @@ Interpreter::endXmlLog()
   xmlLog = 0;
 }
 
+void
+Interpreter::beginLatexLog(const char* fileName)
+{
+  delete latexBuffer;
+  latexBuffer = new MaudeLatexBuffer(fileName);
+}
+
+void
+Interpreter::endLatexLog()
+{
+  delete latexBuffer;
+  latexBuffer = 0;
+}
+
 bool
 Interpreter::setCurrentModule(const Vector<Token>& moduleExpr, int start)
 {
@@ -229,7 +273,7 @@ Interpreter::setCurrentModule(SyntacticPreModule* module)
       if (currentModule != 0)
 	{
 	  clearContinueInfo();
-	  currentModule->loseFocus();
+	  currentModule->loseFocus(getFlag(AUTO_CLEAR_CACHES));
 	}
       currentModule = module;
     }
@@ -272,13 +316,13 @@ Interpreter::makeClean(int lineNumber)
 {
   if (currentModule != 0 && !(currentModule->isComplete()))
     {
-      IssueAdvisory(LineNumber(lineNumber) << ": discarding incomplete module.");
+      IssueAdvisory(*currentModule << ": discarding incomplete module " << QUOTE(currentModule) << ".");
       delete currentModule;
       currentModule = 0;
     }
   else if (currentView != 0 && !(currentView->isComplete()))
     {
-      IssueAdvisory(LineNumber(lineNumber) << ": discarding incomplete view.");
+      IssueAdvisory(*currentView << ": discarding incomplete view " << QUOTE(currentView) << ".");
       delete currentView;
       currentView = 0;
     }
@@ -288,10 +332,8 @@ void
 Interpreter::addSelected(const Vector<Token>& opName)
 {
   selected.insert(Token::bubbleToPrefixNameCode(opName));
-  //  opName.contractTo(0);
 }
 
-
 void
 Interpreter::updateSet(set<int>& target, bool add)
 {
@@ -299,16 +341,26 @@ Interpreter::updateSet(set<int>& target, bool add)
     target.insert(selected.begin(), selected.end());
   else
     {
-      FOR_EACH_CONST(i, set<int>, selected)
-	target.erase(*i);
+      for (int i : selected)
+	target.erase(i);
     }
   selected.clear();
 }
 
-bool
-Interpreter::concealedSymbol(Symbol* symbol)
+void
+Interpreter::printConceal(bool add)
 {
-  return getPrintFlag(PRINT_CONCEAL) && concealedSymbols.find(symbol->id()) != concealedSymbols.end();
+  if (add)
+    {
+      for (int i : selected)
+	insertConcealed(i);
+    }
+  else
+    {
+      for (int i : selected)
+	eraseConcealed(i);
+    }
+  selected.clear();
 }
 
 void
@@ -322,9 +374,19 @@ Interpreter::parse(const Vector<Token>& subject)
   Term* s = currentModule->getFlatModule()->parseTerm(subject);
   if (s != 0)
     {
+      bool showCommand = getFlag(SHOW_COMMAND);
+      if (latexBuffer != 0)
+	latexBuffer->generateCommand(showCommand, "parse", s);
+
       if (s->getSortIndex() == Sort::SORT_UNKNOWN)
 	s->symbol()->fillInSortInfo(s);
       cout << s->getSort() << ": " << s << '\n';
+      if (latexBuffer != 0)
+	{
+	  latexBuffer->generateResult(s);
+	  latexBuffer->cleanUp();
+	}
+
       s->deepSelfDestruct();
     }
 }
@@ -333,19 +395,68 @@ void
 Interpreter::showSortsAndSubsorts() const
 {
   currentModule->getFlatModule()->showSortsAndSubsorts(cout);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show sorts", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowSortsAndSubsorts(latexBuffer->getStream());
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showModule(bool all) const
 {
   currentModule->getFlatModule()->showModule(cout, all);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), all ? "show all" : "show desugared", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowModule(latexBuffer->getStream(), all);
+      latexBuffer->cleanUp();
+    }
+}
+
+void
+Interpreter::showPreModule() const
+{
+  currentModule->showModule(cout);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show module", currentModule->getFlatModule());
+      currentModule->latexShowModule(latexBuffer->getStream());
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showView() const
 {
   if (currentView->evaluate())  // in case it became stale
-    currentView->showView(cout);
+    {
+      currentView->showView(cout);
+      if (latexBuffer)
+	{
+	  latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show view", currentView);
+	  currentView->latexShowView(latexBuffer->getStream());
+	  latexBuffer->cleanUp();
+	}
+    }
+  else
+    IssueWarning("view " << QUOTE(currentView) << " cannot be used due to earlier errors.");
+}
+
+void
+Interpreter::showProcessedView() const
+{
+  if (currentView->evaluate())  // in case it became stale
+    {
+      currentView->showProcessedView(cout);
+      if (latexBuffer)
+	{
+	  latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show processed view", currentView);
+	  currentView->latexShowProcessedView(latexBuffer->getStream());
+	  latexBuffer->cleanUp();
+	}
+    }
   else
     IssueWarning("view " << QUOTE(currentView) << " cannot be used due to earlier errors.");
 }
@@ -356,6 +467,15 @@ Interpreter::showModules(bool all) const
   showNamedModules(cout);
   if (all)
     showCreatedModules(cout);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show modules");
+      ostream& s = latexBuffer->getStream();
+      latexShowNamedModules(s);
+      if (all)
+	latexShowCreatedModules(s);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
@@ -364,6 +484,15 @@ Interpreter::showViews(bool all) const
   showNamedViews(cout);
   if (all)
     showCreatedViews(cout);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show views");
+      ostream& s = latexBuffer->getStream();
+      latexShowNamedViews(s);
+      if (all)
+	latexShowCreatedViews(s);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
@@ -371,54 +500,109 @@ Interpreter::showOps(bool all) const
 {
   currentModule->getFlatModule()->showPolymorphs(cout, false, all);
   currentModule->getFlatModule()->showOps(cout, false, all);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show ops", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowPolymorphs(latexBuffer->getStream(), "", all);
+      currentModule->getFlatModule()->latexShowOps(latexBuffer->getStream(), "", all);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showVars() const
 {
   currentModule->getFlatModule()->showVars(cout, false);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show vars", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowVars(latexBuffer->getStream(), "");
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showMbs(bool all) const
 {
   currentModule->getFlatModule()->showMbs(cout, false, all);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show mbs", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowMbs(latexBuffer->getStream(), "", all);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showEqs(bool all) const
 {
   currentModule->getFlatModule()->showEqs(cout, false, all);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show eqs", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowEqs(latexBuffer->getStream(), "", all);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showRls(bool all) const
 {
   currentModule->getFlatModule()->showRls(cout, false, all);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show rls", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowRls(latexBuffer->getStream(), "", all);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showStrats(bool all) const
 {
   currentModule->getFlatModule()->showStrats(cout, false, all);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show strats", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowStrats(latexBuffer->getStream(), "", all);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showSds(bool all) const
 {
   currentModule->getFlatModule()->showSds(cout, false, all);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show sds", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowSds(latexBuffer->getStream(), "", all);
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showKinds() const
 {
   currentModule->getFlatModule()->showKinds(cout);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show kinds", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowKinds(latexBuffer->getStream());
+      latexBuffer->cleanUp();
+    }
 }
 
 void
 Interpreter::showSummary() const
 {
   currentModule->getFlatModule()->showSummary(cout);
+  if (latexBuffer)
+    {
+      latexBuffer->generateShow(getFlag(SHOW_COMMAND), "show summary", currentModule->getFlatModule());
+      currentModule->getFlatModule()->latexShowSummary(latexBuffer->getStream());
+      latexBuffer->cleanUp();
+    }
 }
 
 ImportModule*
@@ -612,6 +796,10 @@ Interpreter::handleArgument(const ViewExpression* expr,
 ImportModule*
 Interpreter::makeModule(const ModuleExpression* expr, EnclosingObject* enclosingObject)
 {
+  //
+  //	All successfully contructed modules end up in the ModuleCache so we need no worry
+  //	about garbage collecting submodules if we fail part way through a construction.
+  //
   switch (expr->getType())
     {
     case ModuleExpression::MODULE:
@@ -639,11 +827,10 @@ Interpreter::makeModule(const ModuleExpression* expr, EnclosingObject* enclosing
       }
     case ModuleExpression::SUMMATION:
       {
-	const list<ModuleExpression*>& modules = expr->getModules();
 	Vector<ImportModule*> fms;
-	FOR_EACH_CONST(i, list<ModuleExpression*>, modules)
+	for (ModuleExpression* m : expr->getModules())
 	  {
-	    if (ImportModule* fm = makeModule(*i, enclosingObject))
+	    if (ImportModule* fm = makeModule(m, enclosingObject))
 	      {
 		if (fm->hasFreeParameters())
 		  {
