@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2024 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -108,11 +108,13 @@
 MixfixParser::MixfixParser(MixfixModule& client,
 			   bool complexFlag,
 			   int componentNonTerminalBase,
+			   int numberOfTypes,
 			   int nextNonTerminalCode,
 			   int nrTokensGuess)
   : client(client),
     complexParser(complexFlag),
     componentNonTerminalBase(componentNonTerminalBase),
+    numberOfTypes(numberOfTypes),
     tokenSet(nrTokensGuess),
     specialTerminals(Token::LAST_PROPERTY)
 {
@@ -367,7 +369,18 @@ MixfixParser::makeUnifyCommand(Vector<Term*>& lhs, Vector<Term*>& rhs)
 }
 
 void
-MixfixParser::makeSearchCommand(Term*& initial,
+MixfixParser::makeTermDisjunction(int node, Vector<Term*>& terms)
+{
+  for (;; node = parser.getChild(node, 1))
+    {
+      terms.push_back(makeTerm(parser.getChild(node, 0)));
+      if (actions[parser.getProductionNumber(node)].action != MAKE_TERM_DISJUNCTION)
+	break;
+    }
+}
+
+void
+MixfixParser::makeSearchCommand(Vector<Term*>& initial,
 				int& searchType,
 				Term*& target,
 				Vector<ConditionFragment*>& condition)
@@ -375,7 +388,11 @@ MixfixParser::makeSearchCommand(Term*& initial,
   Assert(nrParses > 0, "no parses");
   int node = ROOT_NODE;
   int searchPair = parser.getChild(node, 0);
-  initial = makeTerm(parser.getChild(searchPair, 0));
+
+  makeTermDisjunction(parser.getChild(searchPair, 0), initial);
+
+  //initial.push_back(makeTerm(parser.getChild(searchPair, 0)));  // FIXME
+  
   int arrowType = parser.getChild(searchPair, 1);
   searchType = actions[parser.getProductionNumber(arrowType)].data;
   target = makeTerm(parser.getChild(searchPair, 2));
@@ -1053,6 +1070,11 @@ MixfixParser::makeAttributePart(int node,
 	    flags.setFlags(NARROWING);
 	    break;
 	  }
+	case MAKE_EXTENSION_ATTRIBUTE:
+	  {
+	    flags.setFlags(EXTENSION);
+	    break;
+	  }
 	case MAKE_DNT_ATTRIBUTE:
 	  {
 	    if (client.isObjectOriented())
@@ -1180,6 +1202,9 @@ MixfixParser::makeStatementPart(int node,
 	WarningCheck(!(flags.getFlag(NARROWING)),
 		     LineNumber(lineNumber) <<
 		     ": narrowing attribute not allowed for membership axioms.");
+	WarningCheck(!(flags.getFlag(EXTENSION)),
+		     LineNumber(lineNumber) <<
+		     ": extension attribute not allowed for membership axioms.");
 	Term* lhs = makeTerm(parser.getChild(pairNode, 0));
 	Sort* sort = getSort(parser.getChild(pairNode, 1));
 	SortConstraint* sc = new SortConstraint(label, lhs, sort, condition);
@@ -1212,6 +1237,16 @@ MixfixParser::makeStatementPart(int node,
 	    else
 	      IssueWarning(LineNumber(lineNumber) << ": variant attribute not allowed for conditional equations.");
 	  }
+	if (flags.getFlag(EXTENSION))
+	  {
+	    if (condition.empty())
+	      eq->setExtension();
+	    else
+	      {
+		IssueWarning(LineNumber(lineNumber) <<
+			     ": extension attribute not allowed for conditional equations.");
+	      }
+	  }
 	eq->setLineNumber(lineNumber);
 	client.insertEquation(eq);
 	if (metadata != NONE)
@@ -1242,6 +1277,16 @@ MixfixParser::makeStatementPart(int node,
 	    else
 	      IssueWarning(LineNumber(lineNumber) << ": narrowing attribute not allowed for conditional rules.");
 	  }
+	if (flags.getFlag(EXTENSION))
+	  {
+	    if (condition.empty())
+	      rl->setExtension();
+	    else
+	      {
+		IssueWarning(LineNumber(lineNumber) <<
+			     ": extension attribute not allowed for conditional rules.");
+	      }
+	  }
 	rl->setLineNumber(lineNumber);
 	client.insertRule(rl);
 	if (metadata != NONE)
