@@ -2,7 +2,7 @@
 
     This file is part of the Maude 3 interpreter.
 
-    Copyright 1997-2021 SRI International, Menlo Park, CA 94025, USA.
+    Copyright 1997-2023 SRI International, Menlo Park, CA 94025, USA.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -33,6 +33,8 @@
 #include "core.hh"
 #include "variable.hh"
 #include "strategyLanguage.hh"
+#include "higher.hh"
+#include "strategyLanguage.hh"
 #include "mixfix.hh"
 
 //      interface class definitions
@@ -47,9 +49,32 @@
 #include "strategyDefinition.hh"
 #include "sortConstraint.hh"
 
+//	higher class definitions
+#include "equalityConditionFragment.hh"
+#include "sortTestConditionFragment.hh"
+#include "assignmentConditionFragment.hh"
+#include "rewriteConditionFragment.hh"
+
+//	strategy language class definitions
+#include "trivialStrategy.hh"
+#include "applicationStrategy.hh"
+#include "concatenationStrategy.hh"
+#include "unionStrategy.hh"
+#include "iterationStrategy.hh"
+#include "branchStrategy.hh"
+#include "testStrategy.hh"
+#include "subtermStrategy.hh"
+#include "callStrategy.hh"
+#include "oneStrategy.hh"
+
 //	front end class definitions
 #include "userLevelRewritingContext.hh"
 #include "visibleModule.hh"
+#include "interpreter.hh"
+
+//	our stuff
+#include "latexModulePrint.cc"
+#include "latexStrategyPrint.cc"
 
 VisibleModule::VisibleModule(int name, ModuleType moduleType, Interpreter* owner)
   : ImportModule(name, moduleType),
@@ -91,13 +116,16 @@ VisibleModule::showKinds(ostream& s) const
   for (int i = 0; i < nrKinds; i++)
     {
       const ConnectedComponent* c = kinds[i];
-      s << c->sort(0);
+      s << c->sort(Sort::KIND);
       if (c->errorFree())
 	s << " (error free)";
       s<< ":\n";
       int nrSorts = c->nrSorts();
       for (int j = 1; j < nrSorts; j++)
-	s << '\t' << j << '\t' << c->sort(j) << '\n';
+	{
+	  s << '\t' << j << '\t' << c->sort(j)  << '\n';
+	  DebugInfo(Token::name(c->sort(j)->id()));
+	}
       if (i + 1 < nrKinds)
 	s << '\n';
       DebugAdvisory("match indices = " << c->getLastAllocatedMatchIndex());
@@ -168,13 +196,17 @@ VisibleModule::showSortsAndSubsorts(ostream& s) const
 void
 VisibleModule::showImports(ostream& s) const
 {
+  //
+  //	Imports are always indented.
+  //
   int nrImports = getNrImports();
   for (int i = 0; i < nrImports; i++)
     {
       if (UserLevelRewritingContext::interrupted())
 	return;
-      s << "  " << importModeString(getImportMode(i)) <<
-	' ' << getImportedModule(i) << " .\n";
+      s << "  " << importModeString(getImportMode(i)) << " ";
+      getImportedModule(i)->printModuleExpression(s, false);
+      s << " .\n";
     }
 }
 
@@ -263,11 +295,29 @@ VisibleModule::showVars(ostream& s, bool indent) const
 {
   const char* ind = indent ? "  " : "";
   const AliasMap& variableAliases = getVariableAliases();
-  FOR_EACH_CONST(i, AliasMap, variableAliases)
+  if (owner->getPrintFlag(Interpreter::PRINT_COMBINE_VARS))
     {
-      if (UserLevelRewritingContext::interrupted())
-	return;
-      s << ind << "var " << Token::name(i->first) << " : " << i->second << " .\n";
+      map<int, Vector<int>> reverseMap;
+      for (const auto& p : variableAliases)
+	reverseMap[p.second->getIndexWithinModule()].push_back(p.first);
+
+      const Vector<Sort*>& sorts = getSorts();
+      for (const auto& p : reverseMap)
+	{
+	  s << ind << "var" << pluralize(p.second.size());
+	  for (int v : p.second)
+	    s << " " << Token::name(v);
+	  s << " : " << sorts[p.first] << " .\n";
+	}
+    }
+  else
+    {
+      for (const auto& p : variableAliases)
+	{
+	  if (UserLevelRewritingContext::interrupted())
+	    return;
+	  s << ind << "var " << Token::name(p.first) << " : " << p.second << " .\n";
+	}
     }
 }
 
@@ -357,13 +407,18 @@ VisibleModule::showPolymorphAttributes(ostream& s, int index) const
     s << " comm";
   if (st.hasFlag(SymbolType::LEFT_ID | SymbolType::RIGHT_ID))
     {
-      s << (st.hasFlag(SymbolType::LEFT_ID) ? " left" : " right") <<
-	" id: " << getPolymorphIdentity(index);
+      //
+      //	A polymorphic symbol can only have a left identity or a right identity - not both.
+      //
+      s << (st.hasFlag(SymbolType::LEFT_ID) ? " left" : " right") << " id: ";
+      MixfixModule::prettyPrint(s, getPolymorphIdentity(index), *owner, true);  // assume range kind is known
     }
   if (st.hasFlag(SymbolType::IDEM))
     s << " idem";
   if (st.hasFlag(SymbolType::ITER))
     s << " iter";
+  if (st.hasFlag(SymbolType::PCONST))
+    s << " pconst";
   //
   //	Object-oriented attributes.
   //
@@ -371,6 +426,8 @@ VisibleModule::showPolymorphAttributes(ostream& s, int index) const
     s << " obj";
   if (st.hasFlag(SymbolType::MESSAGE))
     s << " msg";
+  if (st.hasFlag(SymbolType::PORTAL))
+    s << " portal";
   if (st.hasFlag(SymbolType::CONFIG))
     s << " config";
   //
@@ -450,7 +507,8 @@ VisibleModule::showPolymorphAttributes(ostream& s, int index) const
 	}
       s << ')';
     }
-
+  if (st.hasFlag(SymbolType::LATEX))
+    s << " latex (" << Token::name(getPolymorphLatexMacro(index)) << ")";
   int metadata = getPolymorphMetadata(index);
   if (metadata != NONE)
     s << " metadata " << Token::name(metadata);
@@ -458,52 +516,57 @@ VisibleModule::showPolymorphAttributes(ostream& s, int index) const
   if (st.hasSpecial())
     {
       s << " special (";
-      int purpose;
-      {
-	Vector<int> items;
-	for (int i = 0; getPolymorphDataAttachment(index, i, purpose, items); i++)
+      if (owner->getPrintFlag(Interpreter::PRINT_HOOKS))
+	{
+	  int purpose;
 	  {
-	    s << "\n    " << "id-hook " << Token::name(purpose);
-	    int nrItems = items.length();
-	    if (nrItems > 0)
+	    Vector<int> items;
+	    for (int i = 0; getPolymorphDataAttachment(index, i, purpose, items); ++i)
 	      {
-		for (int j = 0; j < nrItems; j++)
-		  s << ((j == 0) ? " (" : " ") << Token::name(items[j]);
-		s << ')';
+		s << "\n    " << "id-hook " << Token::name(purpose);
+		int nrItems = items.length();
+		if (nrItems > 0)
+		  {
+		    for (int j = 0; j < nrItems; j++)
+		      s << ((j == 0) ? " (" : " ") << Token::name(items[j]);
+		    s << ')';
+		  }
 	      }
 	  }
-      }
-      {
-	Symbol* op;
-	for (int i = 0; getPolymorphSymbolAttachment(index, i, purpose, op); i++)
 	  {
-	    s << "\n    " << "op-hook " << Token::name(purpose) << " (" <<
-	      op << " : ";
-	    const Vector<Sort*>& domainAndRange =
-	      op->getOpDeclarations()[0].getDomainAndRange();
-	    int nrSorts = domainAndRange.length() - 1;
-	    //
-	    //	We don't use operator<< for sorts in an op-hook since they
-	    //	should always be printed as single tokens.
-	    //
-	    for (int j = 0; j < nrSorts; j++)
+	    Symbol* op;
+	    for (int i = 0; getPolymorphSymbolAttachment(index, i, purpose, op); ++i)
 	      {
-		Sort* sort = domainAndRange[j];
+		s << "\n    " << "op-hook " << Token::name(purpose) << " (" <<
+		  prettyOpName(op->id(), Token::BARE_COLON) << " : ";
+		const Vector<Sort*>& domainAndRange =
+		  op->getOpDeclarations()[0].getDomainAndRange();
+		int nrSorts = domainAndRange.length() - 1;
+		//
+		//	We don't use operator<< for sorts in an op-hook since they
+		//	should always be printed as single tokens.
+		//
+		for (int j = 0; j < nrSorts; j++)
+		  {
+		    Sort* sort = domainAndRange[j];
+		    if (sort->index() == Sort::KIND)
+		      sort = sort->component()->sort(1);
+		    s << Token::name(sort->id()) << ' ';
+		  }
+		Sort* sort = domainAndRange[nrSorts];
 		if (sort->index() == Sort::KIND)
 		  sort = sort->component()->sort(1);
-		s << Token::name(sort->id()) << ' ';
+		s << "~> " << Token::name(sort->id()) << ')';
 	      }
-	    Sort* sort = domainAndRange[nrSorts];
-	    if (sort->index() == Sort::KIND)
-	      sort = sort->component()->sort(1);
-	    s << "~> " << Token::name(sort->id()) << ')';
 	  }
-      }
-      {
-	Term* term;
-	for (int i = 0; getPolymorphTermAttachment(index, i, purpose, term); i++)
-	  s << "\n    " << "term-hook " << Token::name(purpose) << " (" << term << ")";
-      }
+	  {
+	    Term* term;
+	    for (int i = 0; getPolymorphTermAttachment(index, i, purpose, term); ++i)
+	      s << "\n    " << "term-hook " << Token::name(purpose) << " (" << term << ")";
+	  }
+	}
+      else
+	s << "...";
       s << ')';
     }
 }
@@ -525,7 +588,8 @@ void
 VisibleModule::showPolymorphDecl(ostream& s, bool indent, int index) const
 {
   const char* ind = indent ? "  " : "";
-  s << ind << "op " << getPolymorphName(index) << " :";
+  s << ind << "op " << prettyOpName(getPolymorphName(index).code(), Token::EXPOSED_COLON) << " :";
+  
   const Vector<Sort*>& domainAndRange = getPolymorphDomainAndRange(index);
   int nrArgs = domainAndRange.length() - 1;
   for (int i = 0; i < nrArgs; i++)
@@ -559,9 +623,8 @@ VisibleModule::showPolymorphDecl(ostream& s, bool indent, int index) const
 void
 VisibleModule::showOps(ostream& s, bool indent, bool all) const
 {
-  int begin = all ? 0 : getNrImportedSymbols();
   int end = getNrUserSymbols();
-  for (int i = begin; i < end; i++)
+  for (int i = 0; i < end; ++i)
     {
       if (UserLevelRewritingContext::interrupted())
 	return;
@@ -585,7 +648,7 @@ VisibleModule::showDecls(ostream& s, bool indent, int index, bool all) const
       if (UserLevelRewritingContext::interrupted())
 	return;
       const Vector<Sort*>& dec = opDecls[i].getDomainAndRange();
-      s << ind << "op " << symbol << " :";
+      s << ind << "op " << prettyOpName(symbol->id(), Token::EXPOSED_COLON) << " :";
       for (int j = 0; j < nrArgs; j++)
 	s << ' ' << dec[j];
       s << " -> " << dec[nrArgs];
@@ -633,11 +696,6 @@ VisibleModule::showAttributes(ostream& s, Symbol* symbol, int opDeclIndex) const
       s << space << "comm";
       space = " ";
     }
-  if (st.hasFlag(SymbolType::ITER))
-    {
-      s << space << "iter";
-      space = " ";
-    }
   if (st.hasFlag(SymbolType::LEFT_ID | SymbolType::RIGHT_ID))
     {
       s << space;
@@ -647,26 +705,41 @@ VisibleModule::showAttributes(ostream& s, Symbol* symbol, int opDeclIndex) const
       else if (!(st.hasFlag(SymbolType::RIGHT_ID | SymbolType::COMM)))
 	s << "left ";
       s << "id: ";
-      Term* id = safeCast(BinarySymbol*, symbol)->getIdentity();
+      Term* id = safeCastNonNull<BinarySymbol*>(symbol)->getIdentity();
       if (id != 0)
-	s << id;
+	MixfixModule::prettyPrint(s, id, *owner, true);  // assume range kind is known
     }
   if (st.hasFlag(SymbolType::IDEM))
     {
       s << space << "idem";
       space = " ";
     }
+  if (st.hasFlag(SymbolType::ITER))
+    {
+      s << space << "iter";
+      space = " ";
+    }
+  if (st.hasFlag(SymbolType::PCONST))
+    {
+      s << space << "pconst";
+      space = " ";
+    }
   //
   //	Object-oriented attributes.
   //
+  if (st.hasFlag(SymbolType::OBJECT))
+    {
+      s << space << "obj";
+      space = " ";
+    }
   if (st.hasFlag(SymbolType::MESSAGE))
     {
       s << space << "msg";
       space = " ";
     }
-  if (st.hasFlag(SymbolType::OBJECT))
+  if (st.hasFlag(SymbolType::PORTAL))
     {
-      s << space << "obj";
+      s << space << "portal";
       space = " ";
     }
   if (st.hasFlag(SymbolType::CONFIG))
@@ -755,6 +828,11 @@ VisibleModule::showAttributes(ostream& s, Symbol* symbol, int opDeclIndex) const
 	}
       s << ')';
     }
+  if (st.hasFlag(SymbolType::LATEX))
+    {
+      s << space << "latex (" << Token::name(getLatexMacro(symbol)) << ")";
+      space = " ";
+    }
   if (metadata != NONE)
     {
       s << space << "metadata " << Token::name(metadata);
@@ -763,62 +841,67 @@ VisibleModule::showAttributes(ostream& s, Symbol* symbol, int opDeclIndex) const
   if (st.hasSpecial())
     {
       s << space << "special (";
-      Vector<const char*> purposes;
-      {
-	Vector<Vector<const char*> > data;
-	getDataAttachments(symbol, decl.getDomainAndRange(), purposes, data);
-	int nrHooks = purposes.length();
-	for (int i = 0; i < nrHooks; i++)
+      if (owner->getPrintFlag(Interpreter::PRINT_HOOKS))
+	{
+	  Vector<const char*> purposes;
 	  {
-	    s << "\n    " << "id-hook " << purposes[i];
-	    const Vector<const char*>& items = data[i];
-	    int nrItems = items.length();
-	    if (nrItems > 0)
+	    Vector<Vector<const char*> > data;
+	    getDataAttachments(symbol, decl.getDomainAndRange(), purposes, data);
+	    int nrHooks = purposes.length();
+	    for (int i = 0; i < nrHooks; i++)
 	      {
-		for (int j = 0; j < nrItems; j++)
-		  s << ((j == 0) ? " (" : " ") << items[j];
-		s << ')';
+		s << "\n    " << "id-hook " << purposes[i];
+		const Vector<const char*>& items = data[i];
+		int nrItems = items.length();
+		if (nrItems > 0)
+		  {
+		    for (int j = 0; j < nrItems; j++)
+		      s << ((j == 0) ? " (" : " ") << items[j];
+		    s << ')';
+		  }
 	      }
 	  }
-      }
-      purposes.clear();
-      {
-	Vector<Symbol*> symbols;
-	getSymbolAttachments(symbol, purposes, symbols);
-	int nrHooks = purposes.length();
-	for (int i = 0; i < nrHooks; i++)
+	  purposes.clear();
 	  {
-	    Symbol* op = symbols[i];
-	    s << "\n    " << "op-hook " << purposes[i] << " (" <<
-	      op << " : ";
-	    const Vector<Sort*>& domainAndRange =
-	      op->getOpDeclarations()[0].getDomainAndRange();
-	    int nrSorts = domainAndRange.length() - 1;
-	    //
-	    //	We don't use operator<< for sorts in an op-hook since they
-	    //	should always be printed as single tokens.
-	    //
-	    for (int j = 0; j < nrSorts; j++)
+	    Vector<Symbol*> symbols;
+	    getSymbolAttachments(symbol, purposes, symbols);
+	    int nrHooks = purposes.length();
+	    for (int i = 0; i < nrHooks; i++)
 	      {
-		Sort* sort = domainAndRange[j];
+		Symbol* op = symbols[i];
+		s << "\n    " << "op-hook " << purposes[i] << " (" <<
+		  prettyOpName(op->id(), Token::BARE_COLON) << " : ";
+		const Vector<Sort*>& domainAndRange =
+		  op->getOpDeclarations()[0].getDomainAndRange();
+		int nrSorts = domainAndRange.length() - 1;
+		//
+		//	We don't use operator<< for sorts in an op-hook since they
+		//	should always be printed as single tokens.
+		//
+		for (int j = 0; j < nrSorts; j++)
+		  {
+		    Sort* sort = domainAndRange[j];
+		    if (sort->index() == Sort::KIND)
+		      sort = sort->component()->sort(1);
+		    s << Token::name(sort->id()) << ' ';
+		  }
+		Sort* sort = domainAndRange[nrSorts];
 		if (sort->index() == Sort::KIND)
 		  sort = sort->component()->sort(1);
-		s << Token::name(sort->id()) << ' ';
+		s << "~> " << Token::name(sort->id()) << ')';
 	      }
-	    Sort* sort = domainAndRange[nrSorts];
-	    if (sort->index() == Sort::KIND)
-	      sort = sort->component()->sort(1);
-	    s << "~> " << Token::name(sort->id()) << ')';
 	  }
-      }
-      purposes.clear();
-      {
-	Vector<Term*> terms;
-	getTermAttachments(symbol, purposes, terms);
-	int nrHooks = purposes.length();
-	for (int i = 0; i < nrHooks; i++)
-	  s << "\n    " << "term-hook " << purposes[i] << " (" << terms[i] << ")";
-      }
+	  purposes.clear();
+	  {
+	    Vector<Term*> terms;
+	    getTermAttachments(symbol, purposes, terms);
+	    int nrHooks = purposes.length();
+	    for (int i = 0; i < nrHooks; i++)
+	      s << "\n    " << "term-hook " << purposes[i] << " (" << terms[i] << ")";
+	  }
+	}
+      else
+	s << "...";
       s << ')';
     }
   s << ']';
